/*
 * Corona Stats
 * Copyright (C) 2020, Rahul Kumar Patel <auroraoss.dev@gmail.com>
 *
 * Aurora Store is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 * Corona Stats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Aurora Store.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.aurora.corona.viewmodel;

import android.app.Application;

import androidx.annotation.NonNull;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import com.aurora.corona.Constants;
import com.aurora.corona.model.casetime.CaseReport;
import com.aurora.corona.task.NetworkTask;
import com.aurora.corona.util.Log;
import com.aurora.corona.util.PrefUtil;
import com.google.gson.Gson;

import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;

public class CaseReportModel extends AndroidViewModel {

    private Gson gson = new Gson();
    private MutableLiveData<Boolean> data = new MutableLiveData<>();
    private MutableLiveData<String> error = new MutableLiveData<>();

    public CaseReportModel(@NonNull Application application) {
        super(application);
        fetchOnlineData();
    }

    public MutableLiveData<String> getError() {
        return error;
    }

    public LiveData<Boolean> getData() {
        return data;
    }

    public void fetchOnlineData() {
        Observable.fromCallable(() -> new NetworkTask()
                .get("https://api.covid19india.org/data.json"))
                .subscribeOn(Schedulers.io())
                .map(rawJSON -> gson.fromJson(rawJSON, CaseReport.class))
                .observeOn(AndroidSchedulers.mainThread())
                .doOnNext(this::saveDataToPreferences)
                .doOnError(throwable -> {
                    error.setValue(throwable.getMessage());
                    Log.e(throwable.getMessage());
                })
                .subscribe();
    }

    private void saveDataToPreferences(CaseReport caseReport) {
        try {
            PrefUtil.putString(getApplication(), Constants.PREFERENCE_CASE_TIME_SERIES, gson.toJson(caseReport.getCases_time_series()));
            PrefUtil.putString(getApplication(), Constants.PREFERENCE_KEY_VALUES, gson.toJson(caseReport.getKey_values().get(0)));
            PrefUtil.putString(getApplication(), Constants.PREFERENCE_STATE_WISE, gson.toJson(caseReport.getStatewise()));
            PrefUtil.putString(getApplication(), Constants.PREFERENCE_OVERALL_DATA, gson.toJson(caseReport.getStatewise().get(0)));
            PrefUtil.putString(getApplication(), Constants.PREFERENCE_TESTED, gson.toJson(caseReport.getTested()));
            data.setValue(true);
        } catch (Exception e) {
            data.setValue(false);
            error.setValue(e.getMessage());
        }
    }
}
